(function($, window){
	
SessionManager = function(){
	var 	_timeout = null,
			_warnTime = 120, // 2 min = 120 secs
			_userIdleTimeInMillis = 0,
			_fcmExpireWin,
			_sessionExpiryTimerID,
			_autoLogoff = true,
			_lastUserEvent = new Date();
	
	// if window is unloaded be sure to close the expire window
	$(window).unload(function(){
		try
		{
			if (_fcmExpireWin && !_fcmExpireWin.closed)
				_fcmExpireWin.close();
		}
		catch(e){}
		finally
		{
			// Stop timer
			if(_sessionExpiryTimerID)
				clearTimeout(_sessionExpiryTimerID)
		}
		
	});
	
	var isShowExpiration = function(){
		// only prompt user for expiration if in the top frame of the popup
		// or a window whose opener has been closed.
		return (window == top && (window.name.match("FCMPopupWin.*") || (opener && opener.closed)));
	};
	
	var checkLastUserEvent = function(){
		// This is required to ensure that an opener has not been closed
		// If the window's opener has closed, we need to warn the user in the 
		// current window
		if(!isShowExpiration())
			return;
		
		var now = new Date();
		var idleTime = now - _lastUserEvent;
		if(idleTime >= _userIdleTimeInMillis)
		{
			popExpireWarning();
		}
		else // extend server session just in case user has not made a request to server
		{
			var diff = _userIdleTimeInMillis - idleTime;
			if(diff < 1000) // don't allow less than 1 second
				diff = 1000;
			
//			console.log("Setting timer to "+diff+"ms")
			
			// check back 
			_sessionExpiryTimerID = setTimeout(checkLastUserEvent, diff);
			
			$.ajax({
				async: true,
				type: "POST",
				url: "extendSession.jsp",
				dataType: "html",
				success: function(){},	
				error: popExpireWarning, // if an error occurs, we should show the expire warning
				timeout:5000 // 5 second timeout
			});
		}
	};
	
	// sets the expiry timer to show when warning time period is remaining
	var startExpiryTimer = function(){
		// has the idle time been set
		if(_userIdleTimeInMillis === 0)
		{
			var userIdleTimeInSecs = _timeout - _warnTime;
			if(userIdleTimeInSecs < 0){
				userIdleTimeInSecs = 15; // default to 15 seconds
			}
			_userIdleTimeInMillis = userIdleTimeInSecs * 1000;  // convert to millis
		}
		
		// Stop timer
		if(_sessionExpiryTimerID)
			clearTimeout(_sessionExpiryTimerID)
			
//		console.log("Starting timer: "+_userIdleTimeInMillis + " ms")
		// Restart timer
		_sessionExpiryTimerID = setTimeout(checkLastUserEvent, _userIdleTimeInMillis);
	};
	
	// displays the expiration window to prompt user to extend session on server
	var popExpireWarning = function(){
		// Stop timer and wait for user response
		if(_sessionExpiryTimerID)
			clearTimeout(_sessionExpiryTimerID)
			
		var w=screen.availWidth;
		var h=screen.availHeight;
		var l=(w-440)/2;
		var t=((h-287)/2)-24;
		
		top.onfocus = gotoExpireWin;
		_fcmExpireWin=open("expiration.jsp?autoLogoff="+_autoLogoff,"FCMExpireWin",
					"height=150,width=400,status=no,toolbar=no,menubar=no,location=no,titlebar=no,scrollbars=no,resizable=no,top="+t+",left="+l);
		gotoExpireWin();
	};
	
	// Forces the expiration window to be always on top
	var gotoExpireWin = function(){
		if (_fcmExpireWin)
		{
			if (!_fcmExpireWin.closed)
				_fcmExpireWin.focus();
			else
				top.onfocus = "";
		}
		return false;
	};
	
	// Public methods available to the SessionManager object
	this.reset = function()	{
		startExpiryTimer();
	};
	
	this.getWarnTime = function(){
		return _warnTime;
	};
		
	this.setAutoLogoff = function(autoLogoff){
		_autoLogoff = autoLogoff;
	};
	
	this.setTimeout = function(timeout){
		_timeout = timeout;
		startExpiryTimer();
	};
	
	this.setLastUserEvent = function(){
//		console.log("User Event")
		_lastUserEvent = new Date();
	};
	
}

/**
 * Convenience methods for the iframes to coordinate messages
 */
$.SM = {
	/**
	 * Init sets up the event handlers for the page to communicate user activity to the session manager
	 */
	init: function(options){
		var defaults = $.extend({	timeout: 30,
											autoLogoff: true
										}, options||{});
		
		var userEvents = ['keyup.esfmsm', 
		                  'DOMMouseScroll.esfmsm', 
		                  'mousewheel.esfmsm', 
		                  'mousedown.esfmsm'].join(' ');
		
		var userEventHandler = function(e){
			// Notify the session manager in own window
			top.$('body').data('SessionManager').setLastUserEvent();
			
			// If you have been opened and not the Popup, notify the opener
			if(!top.name.match("FCMPopupWin.*") && opener && !opener.closed) {
				try { opener.top.$('body').data('SessionManager').setLastUserEvent(); } // S1054476
				catch(ex) { /* alert('Suppress security exception: ' + ex); */ }
			}
		};
		
		// Ensures any client-side activity is broadcasted to the top frame
		top.$(document)
			.unbind(userEvents) // Remove existing event handlers
			.bind(userEvents, userEventHandler);
		
		// Attach same handler to each iframe after it loads
		top.$('iframe')
			.unbind('load.esfmsm') // safety precaution
			.bind('load.esfmsm',function(){ 
				$(this).contents().bind(userEvents, userEventHandler)
			});
		
		$.SM.start(defaults.timeout, defaults.autoLogoff);
	},
	
	/**
	 * Create and start the Session Manager
	 */
	start: function(timeout, autoLogoff){ 
		// Only create the session manager object in the top window
		if(typeof timeout === "number")
		{
			// Make sure one hasn't already been created
			var sm = top.$(document.body).data('SessionManager');
			if(!sm)
			{
				sm = new SessionManager();
				top.$(document.body).data('SessionManager', sm);
			}

			if(typeof autoLogoff === "boolean")
				sm.setAutoLogoff(autoLogoff);
			
			sm.setTimeout(timeout);
		}
	},
	
	/**
	 * Reset the Session Manager's timer
	 */
	reset: function(){
		top.$('body').data('SessionManager').reset();
		
		if(!top.name.match("FCMPopupWin.*") && (opener && !opener.closed)) 
			opener.top.$('body').data('SessionManager').reset();
		
		
	},
	
	/**
	 * Get the warn time (in seconds)
	 */
	getWarnTime: function(){
		var windowRef = top;
		if(!top.name.match("FCMPopupWin.*") && (opener && !opener.closed)) 
			windowRef = opener.top;
		
		return windowRef.$('body').data('SessionManager').getWarnTime();
	},
	
	/**
	 * Turn on/off auto logoff feature when the session expires due to inactivity
	 */
	setAutoLogoff: function(autoLogoff){
		top.$('body').data('SessionManager').setAutoLogoff(autoLogoff);
		
		if(!top.name.match("FCMPopupWin.*") && (opener && !opener.closed)) 
			opener.top.$('body').data('SessionManager').setAutoLogoff(autoLogoff);
	}
}

})(jQuery, window);
