

CREATE PROCEDURE MEH_DELETE_DATA (
   IN I_TABLE CHAR(10),
	IN I_MULTIORG_ID	INTEGER,
	IN I_SEG_ID VARCHAR(2),
	IN I_PURGE_DAYS	INTEGER,
	IN I_ROWS_LIMIT	INTEGER,
	IN I_SECONDS_LIMIT DECIMAL(6,0))
	SPECIFIC MEH_DELETE_DATA
	DYNAMIC RESULT SETS 1
	LANGUAGE SQL
	NOT DETERMINISTIC
	EXTERNAL ACTION
	MODIFIES SQL DATA
	OLD SAVEPOINT LEVEL
	P1: BEGIN

		-- processing variables
		DECLARE l_rows_deleted INTEGER DEFAULT 0;
		DECLARE l_elapsed_secs DECIMAL(6,0) DEFAULT 0;
		DECLARE l_rowcount    INTEGER;
    
	   DECLARE l_delDate       date;
		DECLARE l_start_timestamp TIMESTAMP;

		-- return variables
		DECLARE v_sqlstate CHAR(5)DEFAULT '0000';	  -- variable for SQLSTATE
		DECLARE v_sqlcode INTEGER DEFAULT 0;		  -- variable for SQLCODE
		DECLARE v_delete_count INTEGER DEFAULT 0;   -- variable for deleted row count 
      DECLARE v_deldate DATE;                     -- start day of purge
      DECLARE v_multi_org_id INTEGER;             -- return the input parm used
      DECLARE v_seg_id VARCHAR(2);                -- return the input parm used
      DECLARE v_max_rows INTEGER;                 -- return the input parm used
      DECLARE v_max_secs INTEGER;					  -- return the input parm used	
      DECLARE v_rowcount INTEGER;					  -- result of last query
      DECLARE v_exceptions INTEGER DEFAULT 0;	  -- count of entries into error handler
	   DECLARE v_STATE CHAR(5) DEFAULT '00000';    -- sql state after commit
      DECLARE v_USED_SECS DECIMAL(6,0);           -- return elapsed time in seconds

	-- exit reasons: 0=ERROR, 1=row count limit, 2=time limit, 3=EOD	
		DECLARE v_exit_reason INTEGER DEFAULT 3;    -- variable for reason ended processing

		DECLARE SQLCODE INTEGER DEFAULT 0;
		DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

		DECLARE c_result CURSOR WITH RETURN FOR
			SELECT v_exit_reason as EXIT_REASON 
					,v_delete_count as DELETE_ROWCOUNT       	      
					,v_sqlstate as OUT_SQLSTATE
					,v_sqlcode as OUT_SQLCODE
					,v_deldate as PURGE_DATE
					,v_multi_org_id as MULTI_ORG_ID
					,v_seg_id as SEGMENT_ID
					,v_max_rows as DEL_LIMIT
					,v_max_secs as SECS_LIMIT
					,v_rowcount as ROW_COUNT
					,v_exceptions as EXCEPTIONS
					,v_state as EXIT_STATE
					,v_used_secs as TIME
		FROM SYSIBM.SYSDUMMY1;

		DECLARE CONTINUE HANDLER FOR SQLEXCEPTION, SQLWARNING, NOT FOUND
      BEGIN
		    SELECT SQLSTATE,
				     SQLCODE
		    INTO   v_sqlstate,
				     v_sqlcode
	      FROM SYSIBM.SYSDUMMY1;
        SET v_exceptions=v_exceptions+1;
      END;

		-- set lock timeout to NOT WAIT
		SET CURRENT LOCK TIMEOUT NOT WAIT;

		-- 
		-- delete criteria 
		--
		set l_delDate = DATE(((current timestamp - current timezone) - i_purge_days days ));
      set V_deldate = l_delDate;
      set v_multi_org_id = i_multiorg_id;
      set v_seg_id = i_seg_id;
      set v_max_rows = I_ROWS_LIMIT;
      set v_max_secs = I_SECONDS_LIMIT;
		set l_start_timestamp =  current timestamp;

		SET l_rows_deleted = 0; 
		Set l_rowcount=0;		
		SET l_elapsed_secs = l_start_timestamp - current timestamp;
		
	DEL_LOOP:
		REPEAT 

			IF I_SEG_ID='-1' THEN
				IF I_TABLE ='FMH_V_DATA' THEN
					Delete from (Select V_UVR_KEY from fmh_v_data 
						where DATE(V_LAST_UPDT_TS) <= l_delDate and
								v_multi_org_id = I_MULTIORG_ID 
						fetch first 1 rows only);
				ELSE
					Delete from (Select Z_MDL_KEY from fmh_z_data 
						where DATE(Z_LAST_UPDT_TS) <= l_delDate and
								z_multi_org_id = I_MULTIORG_ID 
						fetch first 1 rows only);			
				END IF;
			ELSE
				IF I_TABLE ='FMH_V_DATA' THEN
					Delete from (Select V_UVR_KEY from fmh_v_data 
						where DATE(V_LAST_UPDT_TS) <= l_delDate and
						      v_uvr_id = I_SEG_ID and
								v_multi_org_id = I_MULTIORG_ID 
						fetch first 1 rows only);
				ELSE
					Delete from (Select Z_MDL_KEY from fmh_z_data 
						where DATE(Z_LAST_UPDT_TS) <= l_delDate and
						      z_mdl_id = I_SEG_ID and
								z_multi_org_id = I_MULTIORG_ID 
						fetch first 1 rows only);			
				END IF;			
			END IF;

			-- IGNORE CANNOT LOCK ERROR and do not increment delete counter
			IF v_sqlstate = '40001' or v_sqlstate = '57033' THEN
				SET v_sqlstate = '0000';
				SET v_sqlcode  = 0;
				SET v_rowcount = 0;
			ELSE
				GET DIAGNOSTICS l_rowcount = ROW_COUNT;
				SET v_rowcount=l_rowcount;

				IF ( v_rowcount = 0 ) THEN
					LEAVE DEL_LOOP;

				END IF;

				COMMIT;
			END IF;

			IF v_sqlstate = '0000' THEN
				SET l_rows_deleted = l_rows_deleted + v_rowcount;
				SET v_delete_count = l_rows_deleted;
			ELSE
				SET v_exit_reason = 0;
        		SET V_state=v_sqlstate;
			END IF;

			SET l_elapsed_secs = current timestamp - l_start_timestamp ;

			IF l_rows_deleted = I_ROWS_LIMIT then
				SET v_exit_reason = 1;

			ELSEIF l_elapsed_secs >= I_SECONDS_LIMIT then
				SET v_exit_reason = 2;
				
			END IF;
		
		UNTIL (l_rows_deleted = I_ROWS_LIMIT OR l_elapsed_secs >= I_SECONDS_LIMIT OR v_sqlstate <> '0000')
		
		END REPEAT DEL_LOOP;
    
		IF v_sqlstate <> '0000' THEN
			SET v_exit_reason = 0;
		END IF;
    
		SET v_USED_SECS=l_elapsed_secs;
    
		OPEN c_result;

  END P1
@




